/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetRulePackageCommand.as 4876 2009-08-11 14:22:08Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.command
{
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.utils.getQualifiedClassName;

import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.kerneos.common.event.KerneosNotificationEvent;
import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.rules.common.event.*;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackage;
import org.ow2.jasmine.monitoring.eos.rules.common.model.domain.SynchronizedRulePackageState;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.IRulePackageVO;
import org.ow2.jasmine.monitoring.eos.rules.editor.business.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.event.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.RuleEditorModelLocator;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.domain.RuleEditorModel;

/**
* Get the rule package
* 
* @author Julien Nicoulaud
*/
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")] 
public class GetRulePackageCommand implements ICommand, IResponder
{

    // =========================================================================
    // Execution
    // =========================================================================
    
    /**
    * Send the event to the java side, using the business layer of the pattern
    */
    public function execute(event:CairngormEvent):void
    {
        // Retrieve the component model
        var model:RuleEditorModel = 
            RuleEditorModelLocator.getInstance().ruleEditorModel;
                    
        // Dispatch a notification to Kerneos
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new KerneosNotificationEvent(KerneosNotificationEvent.KERNEOS_NOTIFICATION + model.componentID,
                                         (getQualifiedClassName(this).split("::"))[1] + ": Loading \"" +
                                         (event as RulePackageSystemNameEvent).rulePackageSystemName + "\"",
                                         KerneosNotificationEvent.DEBUG)
        );
                
        // Retrieve the command delegate
        var delegate:IGetRulePackageDelegate = 
        	RuleEditorModelLocator.getInstance().getGetRulePackageDelegate();
        
        // Set this object as a responder
        delegate.responder = this;
        
        // Execute a server-side method thanks to the delegate
        delegate.getRulePackage((event as RulePackageSystemNameEvent).rulePackageSystemName,
                                 model.ruleEngineName);
    }


    // =========================================================================
    // Result handling
    // =========================================================================

    /**
    * Get the result of the java side. this method is called on each event
    * returned from the Java side.
    */
    public function result(event : Object):void
    {
        // Retrieve the model
        var model:RuleEditorModel = 
        	RuleEditorModelLocator.getInstance().ruleEditorModel;
        
        // Retrieve the rule package
        var rulePackage : IRulePackageVO = (event as ResultEvent).result as IRulePackageVO;
        
        // Test if already loaded
        var alreadyEditedRulePackage : SynchronizedRulePackage =
            model.getEditedRulePackage(rulePackage.systemName);
        if (alreadyEditedRulePackage != null) {
        	return;
        }
        
        // Wrap the rule package in a SynchronizedRulePackage
        var newSynchronizedRulePackage : SynchronizedRulePackage =
            new SynchronizedRulePackage(rulePackage,SynchronizedRulePackageState.SYNCHRONIZED);
        
        // Add it if not already done
        model.editedRulePackages.addItem(newSynchronizedRulePackage);
        
        // Launch a test compilation
        CairngormEventDispatcher.getInstance().dispatchEvent(
            new SynchronizedRulePackageEvent(SynchronizedRulePackageEvent.COMPILE_RULE_PACKAGE + model.componentID,
                                             newSynchronizedRulePackage)
        );
    }
    
    
    // =========================================================================
    // Fault handling
    // =========================================================================
    
    /**
    * Handle fault messages
    */
    public function fault(event : Object) : void
    {
        // Retrieve the fault event
        var faultEvent : FaultEvent = FaultEvent(event);

        // Tell the view and let it handle this
        var serverSideExceptionEvent : ServerSideExceptionEvent =
            new ServerSideExceptionEvent(
                "serverSideException",
                new ServerSideException("Error while loading the rule package",
                                        "The operation could not be performed."
                                        + "\n" + faultEvent.fault.faultCode
                                        + "\n" + faultEvent.fault.faultString,
                                        faultEvent.fault.getStackTrace()));
        CairngormEventDispatcher.getInstance().dispatchEvent(serverSideExceptionEvent);
    }

}
}
