/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RulesManagerModelLocator.as 2485 2008-09-30 14:14:35Z nicoulaj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.rules.editor.model.presentation.editor.visual
{
import flash.events.EventDispatcher;
import flash.sampler.*;
import flash.utils.getDefinitionByName;

import mx.binding.utils.*;
import mx.collections.ArrayCollection;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;

import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.ILHSElementVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.IRHSElementVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.LHSCodeSnippetVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.LogicalExpressionVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.RHSCodeSnippetVO;
import org.ow2.jasmine.monitoring.eos.rules.common.vo.rulepackage.visual.RuleVO;
import org.ow2.jasmine.monitoring.eos.rules.editor.event.*;
import org.ow2.jasmine.monitoring.eos.rules.editor.model.domain.RuleEditorModel;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.AddElementChoiceWizard;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.ILHSElementEditor;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.IRHSElementEditor;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.LHSCodeSnippetEditor;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.LogicalExpressionEditor;
import org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.RHSCodeSnippetEditor;
    
/**
* Presentation model for the visual rule editor
* 
* @author Julien Nicoulaud
*/
[Bindable]
public class RuleEditorPM extends EventDispatcher
{
    
    // =========================================================================
    // Variables
    // =========================================================================
    
    /**
    * The edited rule
    */
    public var editedRule : RuleVO;
    
    /**
    * The LHS elements editors
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.ILHSElementEditor')]
    public var lhsEditors : ArrayCollection = new ArrayCollection();
    
    /**
    * The RHS elements editors
    */
    [ArrayElementType('org.ow2.jasmine.monitoring.eos.rules.editor.view.editor.visual.IRHSElementEditor')]
    public var rhsEditors : ArrayCollection = new ArrayCollection();
    
    
    // =========================================================================
    // Constructors
    // =========================================================================
    
    /**
    * Build the Presentation Model for an engine dashboard view
    */
    public function RuleEditorPM(editedRule:RuleVO)
    {
        // Initialize variables
        this.editedRule = editedRule;
        
        // Initialize the editors collection
        initializeLHSEditors();
        initializeRHSEditors();
        
        // Watch the lists changes
        editedRule.lhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,
                                        lhsCollectionChangeHandler);
        editedRule.rhs.addEventListener(CollectionEvent.COLLECTION_CHANGE,
                                        rhsCollectionChangeHandler);
    }
    
    
    // =========================================================================
    // Public methods
    // =========================================================================
    
    /**
    * Create a new LHSElement
    */
    public function addLHSElement():void
    {
        var choiceWizard : AddElementChoiceWizard = new AddElementChoiceWizard('ILHSElementVO');
        choiceWizard.addEventListener(ElementChoiceEvent.CHOOSE_ELEMENT_TYPE,addLHSElementChosen);
    }
    
    /**
    * Create a new RHSElement
    */
    public function addRHSElement():void
    {
        var choiceWizard : AddElementChoiceWizard = new AddElementChoiceWizard('IRHSElementVO');
        choiceWizard.addEventListener(ElementChoiceEvent.CHOOSE_ELEMENT_TYPE,addRHSElementChosen);
    }
    
    
    // =========================================================================
    // Private methods
    // =========================================================================
    
    /**
    * Initialize the collection of LHS editors
    */
    private function initializeLHSEditors():void
    {
        lhsEditors.removeAll();
        for each (var lhsElement : ILHSElementVO in editedRule.lhs) {
            lhsEditors.addItem(createNewLHSEditor(lhsElement));
        }
    }
    
    /**
    * Initialize the collection of RHS editors
    */
    private function initializeRHSEditors():void
    {
        rhsEditors.removeAll();
        for each (var rhsElement : IRHSElementVO in editedRule.rhs) {
            rhsEditors.addItem(createNewRHSEditor(rhsElement));
        }
    }
    
    /**
    * Create a new editor for a LHS element
    */
    private function createNewLHSEditor(lhsElement:ILHSElementVO):ILHSElementEditor
    {
        // Declare a new editor
        var newEditor : ILHSElementEditor;
        
        // Instanciate an editor depending on the type of the object
        if (lhsElement is LHSCodeSnippetVO) {
            newEditor = new LHSCodeSnippetEditor();
        } else if (lhsElement is LogicalExpressionVO) {
            newEditor = new LogicalExpressionEditor();
        } else {
            throw new Error('No editor available for this kind of object');
        }
        newEditor.editedLHSElement = lhsElement;
        
        // Listen to demands from the editor
        newEditor.addEventListener("deleteMe",deleteMeHandler);

        return newEditor;
    }
    
    /**
    * Create a new editor for a RHS element
    */
    private function createNewRHSEditor(rhsElement:IRHSElementVO):IRHSElementEditor
    {
        // Declare a new editor
        var newEditor : IRHSElementEditor;
        
        // Instanciate an editor depending on the type of the object
        if (rhsElement is RHSCodeSnippetVO) {
            newEditor = new RHSCodeSnippetEditor();
        } else {
            throw new Error('No editor available for this kind of object' + rhsElement);
        }
        newEditor.editedRHSElement = rhsElement;
        
        // Listen to demands from the editor
        newEditor.addEventListener("deleteMe",deleteMeHandler);

        return newEditor;
    }
    
    
    // =========================================================================
    // Event handlers
    // =========================================================================
    
    /**
    * When the LHS elements collection changes
    */
    private function lhsCollectionChangeHandler(event:CollectionEvent):void
    {
        switch(event.kind) {
            case CollectionEventKind.ADD:
               for each (var newLhsElement : ILHSElementVO in event.items) {
                   lhsEditors.addItem(createNewLHSEditor(newLhsElement));
               }
               break;
            case CollectionEventKind.REMOVE:
                for each (var removedLhsElement : ILHSElementVO in event.items) {
                    for each (var editor : ILHSElementEditor in lhsEditors) {
                        if (editor.editedLHSElement == removedLhsElement) {
                            lhsEditors.removeItemAt(lhsEditors.getItemIndex(editor));
                        }
                    }
                }
                break;
            default:
                break;
        }
    }
    
    /**
    * When the RHS elements collection changes
    */
    private function rhsCollectionChangeHandler(event:CollectionEvent):void
    {
        switch(event.kind) {
            case CollectionEventKind.ADD:
               for each (var newRhsElement : IRHSElementVO in event.items) {
                   rhsEditors.addItem(createNewRHSEditor(newRhsElement));
               }
               break;
            case CollectionEventKind.REMOVE:
                for each (var removedRhsElement : IRHSElementVO in event.items) {
                    for each (var editor : IRHSElementEditor in rhsEditors) {
                        if (editor.editedRHSElement == removedRhsElement) {
                            rhsEditors.removeItemAt(rhsEditors.getItemIndex(editor));
                        }
                    }
                }
                break;
            default:
                break;
        }
    }
    
    /**
    * When an element type has been chosen to be added to the LHS
    */
    private function addLHSElementChosen(event:ElementChoiceEvent):void
    {
        var newOperandClass:Class = getDefinitionByName(event.elementType) as Class;
        editedRule.lhs.addItem(new newOperandClass());
    }
    
    /**
    * When an element type has been chosen to be added to the RHS
    */
    private function addRHSElementChosen(event:ElementChoiceEvent):void
    {
        var newOperandClass:Class = getDefinitionByName(event.elementType) as Class;
        editedRule.rhs.addItem(new newOperandClass());
    }
        
    /**
    * When a child asks for deletion
    */
    private function deleteMeHandler(event:Event):void
    {
        var targetEditor : Object = event.target;
        if (targetEditor is ILHSElementEditor) {
            editedRule.lhs.removeItemAt(editedRule.lhs.getItemIndex(
                (targetEditor as ILHSElementEditor).editedLHSElement)
            );
        } else if (targetEditor is IRHSElementEditor) {
            editedRule.rhs.removeItemAt(editedRule.rhs.getItemIndex(
                (targetEditor as IRHSElementEditor).editedRHSElement)
            );
        }
    }
}
}
